@@ -20,15 +20,21 @@ from collections import defaultdict
 import warnings
 from functools import partial
 import gc
+import logging
 
 from .utils import (MODEL_CLASSES, is_module_installed, InvalidBenchmark,
                     TwolabelTrainer)
 
 
+logger = logging.getLogger(__name__)
+
+
 class BaseBenchmark(ABC):
     '''Abstract base class for evaluating models.
 
     Args:
+        name (str):
+            The name of the dataset.
         task (str):
             The type of task to be benchmarked.
         num_labels (int or None, optional):
@@ -76,7 +82,9 @@ class BaseBenchmark(ABC):
             If `batch_size` is not among 1, 2, 4, 8, 16 or 32.
     '''
     def __init__(self,
+                 name: str,
                  task: str,
+                 metric_names: Dict[str, str],
                  num_labels: Optional[int] = None,
                  id2label: Optional[List[str]] = None,
                  evaluate_train: bool = False,
@@ -93,6 +101,8 @@ class BaseBenchmark(ABC):
                             '16 or 32.')
         self.batch_size = batch_size
         self.gradient_accumulation = 32 // batch_size
+        self.name = name
+        self.metric_names = metric_names
         self.evaluate_train = evaluate_train
         self.task = task
         self.cache_dir = cache_dir
@@ -338,9 +348,9 @@ class BaseBenchmark(ABC):
         '''
         pass
 
-    @abstractmethod
     def _log_metrics(self,
                      metrics: Dict[str, List[Dict[str, float]]],
+                     finetuned: bool,
                      model_id: str):
         '''Log the metrics.
 
@@ -349,11 +359,36 @@ class BaseBenchmark(ABC):
                 The metrics that are to be logged. This is a dict with keys
                 'train' and 'test', with values being lists of dictionaries
                 full of metrics.
+            ta
             model_id (str):
                 The full HuggingFace Hub path to the pretrained transformer
                 model.
         '''
-        pass
+        # Initial logging message
+        if finetuned:
+            msg = (f'Finished finetuning and evaluation of {model_id} on '
+                   f'{self.name}.')
+        else:
+            msg = (f'Finished evaluation of {model_id} on {self.name}.')
+        logger.info(msg)
+
+        # Logging of the metric(s)
+        for metric_key, metric_name in self.metric_names.items():
+            scores = self._get_stats(metrics, metric_key)
+            test_score, test_se = scores['test']
+            test_score *= 100
+            test_se *= 100
+
+            msg = (f'{metric_name}:\n'
+                   f'  - Test: {test_score:.2f} ± {test_se:.2f}')
+
+            if 'train' in scores.keys():
+                train_score, train_se = scores['train']
+                train_score *= 100
+                train_se *= 100
+                msg += f'\n  - Train: {train_score:.2f} ± {train_se:.2f}'
+
+            logger.info(msg)
 
     @abstractmethod
     def _get_spacy_predictions_and_labels(self,
@@ -456,6 +491,9 @@ class BaseBenchmark(ABC):
         task = model_metadata['task']
         model_dict = self._load_model(model_id, **model_metadata)
 
+        # Define variable that determines if the model should be finetuned
+        finetune = (task == 'fill-mask')
+
         # Load the dataset
         train, test = self._load_data()
 
@@ -463,7 +501,7 @@ class BaseBenchmark(ABC):
         rng = np.random.default_rng(4242)
 
         # Get bootstrap sample indices
-        if task == 'fill-mask' or self.evaluate_train:
+        if finetune or self.evaluate_train:
             train_bidxs = rng.integers(0, len(train),
                                        size=(num_finetunings, len(train)))
         test_bidxs = rng.integers(0, len(test), size=(10, len(test)))
@@ -479,24 +517,24 @@ class BaseBenchmark(ABC):
                 params = dict(framework=framework,
                               config=model.config,
                               tokenizer=tokenizer)
-                if task == 'fill-mask' or self.evaluate_train:
-                    preprocessed_train = self._preprocess_data(train, **params)
-                preprocessed_test = self._preprocess_data(test, **params)
+                if finetune or self.evaluate_train:
+                    train = self._preprocess_data(train, **params)
+                test = self._preprocess_data(test, **params)
             except ValueError:
                 raise InvalidBenchmark('Preprocessing of the dataset could '
                                        'not be done.')
 
             # Get bootstrapped datasets
-            if task == 'fill-mask' or self.evaluate_train:
-                trains = [preprocessed_train]
-                trains = trains + [preprocessed_train[train_bidxs[idx]]
-                                   for idx in range(num_finetunings - 1)]
-            tests = [preprocessed_test]
-            tests = tests + [preprocessed_test[test_bidxs[idx]]
-                             for idx in range(10)]
+            trains = [train]
+            if finetune or self.evaluate_train:
+                trains += [Dataset.from_dict(train[train_bidxs[idx]])
+                           for idx in range(num_finetunings - 1)]
+            tests = [test]
+            tests += [Dataset.from_dict(test[test_bidxs[idx]])
+                      for idx in range(test_bidxs.shape[0])]
 
             # Set up progress bar
-            if task == 'fill-mask':
+            if finetune:
                 if progress_bar:
                     itr = tqdm(range(num_finetunings))
                 else:
@@ -516,15 +554,16 @@ class BaseBenchmark(ABC):
             training_args = TrainingArguments(
                 output_dir='.',
                 evaluation_strategy='epoch',
-                logging_strategy='epoch',
-                save_strategy='no',
+                logging_strategy='epoch' if self.verbose else 'no',
+                save_strategy='epoch',
                 report_to='none',
-                save_total_limit=0,
+                save_total_limit=1,
                 per_device_train_batch_size=self.batch_size,
                 learning_rate=self.learning_rate,
                 num_train_epochs=self.epochs,
                 warmup_steps=self.warmup_steps,
-                gradient_accumulation_steps=self.gradient_accumulation
+                gradient_accumulation_steps=self.gradient_accumulation,
+                load_best_model_at_end=True
             )
 
             # Disable `transformers` verbosity again
@@ -546,10 +585,11 @@ class BaseBenchmark(ABC):
                         )
 
                         # Initialise Trainer
+                        split = trains[idx].train_test_split(0.1, seed=4242)
                         trainer_args = dict(model=model,
                                             args=training_args,
-                                            train_dataset=trains[idx],
-                                            eval_dataset=tests[0],
+                                            train_dataset=split['train'],
+                                            eval_dataset=split['test'],
                                             tokenizer=tokenizer,
                                             data_collator=data_collator,
                                             compute_metrics=compute_metrics)
@@ -565,7 +605,7 @@ class BaseBenchmark(ABC):
                             trainer.remove_callback(PrinterCallback)
 
                         # Finetune the model
-                        if task == 'fill-mask':
+                        if finetune:
                             trainer.train()
 
                         # Log training metrics and save the state
@@ -577,9 +617,9 @@ class BaseBenchmark(ABC):
                             metrics['train'].append(train_metrics)
 
                         # Log test metrics
-                        for test_idx in range(10):
+                        for dataset in tests:
                             test_metrics = trainer.evaluate(
-                                tests[test_idx],
+                                dataset,
                                 metric_key_prefix='test'
                             )
                             metrics['test'].append(test_metrics)
@@ -598,11 +638,15 @@ class BaseBenchmark(ABC):
                         training_args.gradient_accumulation_steps = ga * 2
                         trainer.args = training_args
 
-            self._log_metrics(metrics, model_id=model_id)
+                        # Garbage collection, to avoid memory issues
+                        del model, model_dict
+                        gc.collect()
+
+            self._log_metrics(metrics, model_id=model_id, finetuned=finetune)
 
             # Garbage collection, to avoid memory issues
             del model, model_dict
-            del preprocessed_train, preprocessed_test, train, test
+            del train, test
             gc.collect()
 
             return metrics
@@ -612,11 +656,10 @@ class BaseBenchmark(ABC):
             model = model_dict['model']
 
             # Preprocess the test datasets
-            params = dict(framework=framework)
-            preprocessed_test = self._preprocess_data(test, **params)
-            tests = [preprocessed_test]
-            tests = tests + [preprocessed_test[test_bidxs[idx]]
-                             for idx in range(10)]
+            test = self._preprocess_data(test, framework=framework)
+            tests = [test]
+            tests += [Dataset.from_dict(test[test_bidxs[idx]])
+                      for idx in range(test_bidxs.shape[0])]
 
             # Get the test predictions
             all_test_metrics = list()
@@ -633,15 +676,15 @@ class BaseBenchmark(ABC):
                 all_test_metrics.append(test_metrics)
             metrics = dict(test=all_test_metrics)
 
-            # Preprocess the train datasets
             if self.evaluate_train:
-                preprocessed_train = self._preprocess_data(train, **params)
-                trains = [preprocessed_train]
-                trains = trains + [preprocessed_train[train_bidxs[idx]]
-                                   for idx in range(num_finetunings - 1)]
 
-            # Get the train predictions
-            if self.evaluate_train:
+                # Preprocess the train datasets
+                train = self._preprocess_data(train, framework=framework)
+                trains = [train]
+                trains += [Dataset.from_dict(train[train_bidxs[idx]])
+                           for idx in range(num_finetunings - 1)]
+
+                # Get the train predictions
                 all_train_metrics = list()
                 for dataset in trains:
                     preds_labels = self._get_spacy_predictions_and_labels(
@@ -652,6 +695,7 @@ class BaseBenchmark(ABC):
                     train_metrics = self._compute_metrics(preds_labels)
                     train_metrics = {f'train_{key}': val
                                      for key, val in train_metrics.items()}
+
                 all_train_metrics.append(train_metrics)
                 metrics['train'] = all_train_metrics
 
