@@ -5,7 +5,7 @@ from datasets import Dataset, load_metric
 from functools import partial
 import logging
 from abc import ABC
-from typing import Optional
+from typing import Optional, Dict
 
 from .base import BaseBenchmark
 from .utils import InvalidBenchmark
@@ -45,6 +45,8 @@ class TextClassificationBenchmark(BaseBenchmark, ABC):
         id2label (dict): Conversion dict from NER label indices to the labels.
     '''
     def __init__(self,
+                 name: str,
+                 metric_names: Dict[str, str],
                  id2label: list,
                  epochs: int,
                  cache_dir: str = '.benchmark_models',
@@ -57,6 +59,8 @@ class TextClassificationBenchmark(BaseBenchmark, ABC):
         self._metric = load_metric('f1')
         super().__init__(task='text-classification',
                          num_labels=len(id2label),
+                         name=name,
+                         metric_names=metric_names,
                          id2label=id2label,
                          cache_dir=cache_dir,
                          learning_rate=learning_rate,
